@@ -37,7 +37,7 @@ overlap <- function(object,level=0.95,debias=TRUE,...)
   # fix diagonals
   diag(OVER[,,1]) <- diag(OVER[,,2]) <- diag(OVER[,,3]) <- 1
 
-  dimnames(OVER) <- list(names(object),names(object),c("low","ML","high"))
+  dimnames(OVER) <- list(names(object),names(object),NAMES.CI)
 
   return(OVER)
   # utils::getS3method("overlap",CLASS)(object,...)
@@ -170,7 +170,7 @@ overlap.ctmm <- function(object,level=0.95,debias=TRUE,COV=TRUE,...)
 
     # transform from (square) distance to overlap measure
     CI <- exp(-rev(CI))
-    names(CI) <- c("low","ML","high")
+    names(CI) <- NAMES.CI
 
     return(CI)
   }
@@ -228,108 +228,11 @@ overlap.UD <- function(object,level=0.95,debias=TRUE,...)
   else
   { OVER <- c(NA,OVER,NA) }
 
-  names(OVER) <- c("low","ML","high")
+  names(OVER) <- NAMES.CI
   return(OVER)
 }
 
 
-# CALCULATE A BUNCH OF AKDE UDs ON THE SAME GRID
-# (C) Kevin Winner & C.H. Fleming (2016)
-akde.list <- function(data,CTMM,VMM=NULL,debias=TRUE,smooth=TRUE,error=0.001,res=10,grid=NULL,...)
-{
-  axes <- CTMM[[1]]$axes
-  COL <- length(axes)
-
-  n.instances <- length(data)
-
-  #initialize per-instance vars that need to be tracked
-  HP.list   <- vector("list", n.instances)
-  H.list    <- vector("list", n.instances)
-  UD.list   <- vector("list", n.instances)
-  n.samples.total <- 0
-
-  #step 1: compute the optimal bandwidths for each instance
-  # if (!silent)
-  #  cat("Computing optimal bandwidths\n")
-  for(i.instance in 1:n.instances)
-  {
-    # smooth data
-    if(CTMM[[i.instance]]$error && smooth)
-    {
-      data[[i.instance]] <- predict(CTMM[[i.instance]],data=data[[i.instance]],t=data[[i.instance]]$t)
-      CTMM[[i.instance]]$error <- FALSE # smoothed error model (approximate)
-    }
-
-    #alias for readability
-    instance <- data[[i.instance]]
-    n.samples <- length(instance$x)
-    n.samples.total <- n.samples.total + n.samples
-
-    # weights array argument (feature request from Dong)
-    ARGS <- list(...)
-    weights <- ARGS$weights
-    if(is.null(weights))
-    { weights <- array(FALSE,n.instances) }
-    else
-    {
-      weights <- rep(weights,n.instances)
-      ARGS <- ARGS[names(ARGS)!="weights"]
-    }
-
-    #bandwidth calculations for this instance
-    # HP.list[[i.instance]] <- bandwidth(data = instance, CTMM = CTMM[[i.instance]], VMM=VMM[[i.instance]], verbose = TRUE,...)
-    HP.list[[i.instance]] <- do.call(bandwidth,c(list(data=instance,CTMM=CTMM[[i.instance]],VMM=VMM[[i.instance]],weights=weights[i.instance],verbose=TRUE),ARGS))
-    H.list[[i.instance]]  <- prepare.H(HP.list[[i.instance]]$H,length(instance$t))
-    #reshape H
-    H.list[[i.instance]]  <- array(H.list[[i.instance]], c(n.samples, COL * COL)) #TODO: magic numbers
-  }
-
-  #step 2: compute a grid for all UDs
-  # if (!silent)
-  #  cat("Computing the grid\n")
-  H.all <- do.call("rbind", H.list)
-  H.all <- array(H.all, c(n.samples.total, COL, COL))
-
-  # take only the necessary columns
-  data.all <- lapply(data,function(d) { d[,c("t",axes)] })
-  data.all <- do.call("rbind", data.all)
-
-  dr <- vector("numeric", COL)
-  dr[1] <- sqrt(min(sapply(HP.list, function(i) { i$H[1,1] })))
-  dr[2] <- sqrt(min(sapply(HP.list, function(i) { i$H[2,2] })))
-  if(COL==3) { dr[3] <- sqrt(min(sapply(HP.list, function(i) { i$H[3,3] }))) }
-  dr    <- dr / res
-
-  grid    <- kde.grid(data.all, H.all, alpha = error, dr = dr)
-  dH.orig <- grid$dH
-
-  #now align all UDs to the grid
-  sample.position <- 1 #track position of next sample to read out
-  for (i.instance in 1:n.instances)
-  {
-    # if (!silent)
-    #  cat(sprintf("Aligning UD for instance %d/%d\n", i.instance, n.instances))
-
-    #repeated work, but cheap and more readable than storing from step 1
-    instance <- data[[i.instance]]
-    n.samples <- length(instance$x)
-
-    #remember when we reshaped H? yeah undo that
-    H.list[[i.instance]] <- array(H.list[[i.instance]], c(n.samples, COL, COL))
-
-    grid$dH <- dH.orig[sample.position:(sample.position + n.samples - 1),]
-
-    UD.list[[i.instance]] <- akde(data[[i.instance]],CTMM=HP.list[[i.instance]],debias=debias,smooth=smooth,error=error,res=res,grid=grid,...)
-    attr(UD.list[[i.instance]],"CTMM") <- CTMM[[i.instance]]
-
-    sample.position <- sample.position + n.samples
-  }
-
-  names(UD.list) <- names(data)
-  return(UD.list)
-}
-
-
 ###################
 # average aligned UDs
 mean.UD <- function(x,...)
